package com.qmth.boot.core.cache.config;

import com.github.benmanes.caffeine.cache.Caffeine;
import org.springframework.boot.autoconfigure.condition.ConditionalOnMissingBean;
import org.springframework.cache.CacheManager;
import org.springframework.cache.caffeine.CaffeineCacheManager;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import javax.annotation.Resource;

/**
 * 自定义缓存配置，缺省启用Caffeine作为单实例缓存实现
 */
@Configuration
public class CacheManagerConfig {

    @Resource
    private CacheConfig cacheConfig;

    @Bean
    @ConditionalOnMissingBean(CacheManager.class)
    public CacheManager cacheManager() {
        CaffeineCacheManager cacheManager = new CaffeineCacheManager();
        cacheManager.setAllowNullValues(cacheConfig.isAllowNullValue());
        //默认开启softValues，避免JVM撑爆
        Caffeine<Object, Object> caffeine = Caffeine.newBuilder().softValues();
        //写入失效时长大于0毫秒时才启用
        if (cacheConfig.getExpireAfterWrite().toMillis() > 0) {
            caffeine = caffeine.expireAfterWrite(cacheConfig.getExpireAfterWrite());
        }
        cacheManager.setCaffeine(caffeine);
        return cacheManager;
    }
}
