package com.qmth.boot.core.concurrent.service.impl;

import com.qmth.boot.core.concurrent.service.ConcurrentService;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReadWriteLock;
import java.util.concurrent.locks.ReentrantLock;
import java.util.concurrent.locks.ReentrantReadWriteLock;

@Service
public class MemoryConcurrentService implements ConcurrentService {

    private final Map<String, ReentrantLock> lockMap = new HashMap<>();

    private final Map<String, ReentrantReadWriteLock> readWrite = new HashMap<>();

    @Override
    public Lock getLock(String name) {
        Lock lock = lockMap.get(name);
        if (lock == null) {
            synchronized (lockMap) {
                lock = lockMap.computeIfAbsent(name, key -> new ReentrantLock());
            }
        }
        return lock;
    }

    @Override
    public ReadWriteLock getReadWriteLock(String name) {
        ReadWriteLock lock = readWrite.get(name);
        if (lock == null) {
            synchronized (readWrite) {
                lock = readWrite.computeIfAbsent(name, key -> new ReentrantReadWriteLock());
            }
        }
        return lock;
    }
}
