package com.qmth.boot.core.metrics.service.impl;

import com.qmth.boot.core.metrics.config.MetricsConfig;
import com.qmth.boot.core.metrics.model.MetricsResult;
import com.qmth.boot.core.metrics.service.MetricsService;
import org.springframework.boot.autoconfigure.condition.ConditionalOnMissingBean;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Configuration
public class MemoryMetricsService implements MetricsService {

    private Map<String, MetricsRecorder> recorderMap;

    @Resource
    private MetricsConfig metricsConfig;

    @Bean
    @ConditionalOnMissingBean(MetricsService.class)
    public MetricsService metricsService() {
        return new MemoryMetricsService();
    }

    public MemoryMetricsService() {
        this.recorderMap = new HashMap<>();
    }

    public MetricsConfig getMetricsConfig() {
        return metricsConfig;
    }

    public void setMetricsConfig(MetricsConfig metricsConfig) {
        this.metricsConfig = metricsConfig;
    }

    @Override
    public void record(String endpoint, int status, long timecost) {
        recorderMap.computeIfAbsent(endpoint, key -> new MetricsRecorder(metricsConfig)).record(status, timecost);
    }

    @Override
    public MetricsResult result(String endpoint) {
        MetricsResult result = new MetricsResult();
        MetricsRecorder recorder = recorderMap.get(endpoint);
        if (recorder != null) {
            result = recorder.getResult();
        }
        result.setEndpoint(endpoint);
        return result;
    }

    @Override
    public List<MetricsResult> result() {
        List<MetricsResult> list = new ArrayList<>();
        for (Map.Entry<String, MetricsRecorder> entry : recorderMap.entrySet()) {
            MetricsResult result = entry.getValue().getResult();
            result.setEndpoint(entry.getKey());
            list.add(result);
        }
        return list;
    }

    @Override
    public void reset(String endpoint) {
        recorderMap.remove(endpoint);
    }

    @Override
    public void reset() {
        recorderMap.clear();
    }
}
