package com.qmth.boot.mysql.config;

import ch.qos.logback.classic.Level;
import com.baomidou.mybatisplus.annotation.DbType;
import com.baomidou.mybatisplus.autoconfigure.ConfigurationCustomizer;
import com.baomidou.mybatisplus.autoconfigure.MybatisPlusPropertiesCustomizer;
import com.baomidou.mybatisplus.extension.plugins.MybatisPlusInterceptor;
import com.baomidou.mybatisplus.extension.plugins.inner.BlockAttackInnerInterceptor;
import com.baomidou.mybatisplus.extension.plugins.inner.PaginationInnerInterceptor;
import com.qmth.boot.core.logger.context.LoggerContextService;
import com.qmth.boot.mysql.logger.MybatisPlusLogger;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import javax.annotation.Resource;

/**
 * MyBatisPlus插件配置
 */
@Configuration
public class MybatisPlusAutoConfiguration {

    @Resource
    private MysqlConfig mysqlConfig;

    @Resource
    private LoggerContextService loggerContextService;

    @Bean
    public MybatisPlusInterceptor mybatisPlusInterceptor() {
        MybatisPlusInterceptor interceptor = new MybatisPlusInterceptor();
        interceptor.addInnerInterceptor(new PaginationInnerInterceptor(DbType.MYSQL));
        interceptor.addInnerInterceptor(new BlockAttackInnerInterceptor());
        return interceptor;
    }

    @Bean
    public MybatisPlusPropertiesCustomizer plusPropertiesCustomizer() {
        return plusProperties -> plusProperties.getGlobalConfig().setBanner(false);
    }

    @Bean
    public ConfigurationCustomizer configurationCustomizer() {
        return configuration -> {
            loggerContextService.setLevel(MybatisPlusLogger.class, Level.toLevel(mysqlConfig.getLogLevel()));
            configuration.setLogImpl(MybatisPlusLogger.class);
        };
    }
}
