package com.qmth.boot.redis.cache;

import com.qmth.boot.core.cache.config.CacheConfig;
import com.qmth.boot.redis.constant.RedisConstant;
import org.springframework.cache.CacheManager;
import org.springframework.cache.annotation.CachingConfigurerSupport;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.data.redis.cache.RedisCacheConfiguration;
import org.springframework.data.redis.cache.RedisCacheManager;
import org.springframework.data.redis.connection.RedisConnectionFactory;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.serializer.RedisSerializationContext;

import javax.annotation.Resource;

@Configuration
public class RedisCacheAutoConfiguration extends CachingConfigurerSupport implements RedisConstant {

    @Resource
    private RedisConnectionFactory redisConnectionFactory;

    @Resource
    private RedisTemplate<String, Object> redisTemplate;

    @Resource
    private CacheConfig cacheConfig;

    @Bean
    public CacheManager cacheManager() {
        RedisCacheConfiguration config = RedisCacheConfiguration.defaultCacheConfig()
                //固定key格式与前缀
                .computePrefixWith(key -> CACHE_PREFIX + KEY_SEPARATOR + key + KEY_SEPARATOR)
                //序列化方法
                .serializeKeysWith(
                        RedisSerializationContext.SerializationPair.fromSerializer(redisTemplate.getStringSerializer()))
                .serializeValuesWith(
                        RedisSerializationContext.SerializationPair.fromSerializer(redisTemplate.getValueSerializer()));
        //禁止空值保存
        if (!cacheConfig.isAllowNullValue()) {
            config = config.disableCachingNullValues();
        }
        //写入失效时长大于0毫秒时才启用
        if (cacheConfig.getExpireAfterWrite().toMillis() > 0) {
            config = config.entryTtl(cacheConfig.getExpireAfterWrite());
        }
        return RedisCacheManager.builder(redisConnectionFactory).cacheDefaults(config).build();
    }
}
