package com.qmth.boot.redis.rateLimit;

import com.qmth.boot.core.rateLimit.config.RateLimitRule;
import com.qmth.boot.core.rateLimit.config.RateLimitScope;
import com.qmth.boot.core.rateLimit.config.RateLimitTarget;
import com.qmth.boot.core.rateLimit.service.RateLimitService;
import com.qmth.boot.redis.constant.RedisConstant;
import org.apache.commons.lang3.ArrayUtils;
import org.redisson.api.RRateLimiter;
import org.redisson.api.RateIntervalUnit;
import org.redisson.api.RateType;
import org.redisson.api.RedissonClient;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.concurrent.TimeUnit;

@Service
public class RedisRateLimitService implements RateLimitService, RedisConstant {

    @Resource
    private RedissonClient redissonClient;

    @Override
    public boolean accept(String endpoint, String device, RateLimitRule... rules) {
        if (ArrayUtils.isEmpty(rules)) {
            return true;
        }
        for (RateLimitRule rule : rules) {
            if (!accept(endpoint, device, rule)) {
                return false;
            }
        }
        return true;
    }

    protected boolean accept(String endpoint, String device, RateLimitRule rule) {
        String key = RATE_LIMIT_PREFIX.concat(KEY_SEPARATOR).concat(endpoint);
        if (rule.getTarget() == RateLimitTarget.DEVICE) {
            // 若限流对象是单个设备，则KEY需要包含设备标识
            key = key.concat(KEY_SEPARATOR).concat(device);
        }
        RRateLimiter limiter = redissonClient.getRateLimiter(key);
        limiter.trySetRate(rule.getScope() == RateLimitScope.INSTANCE ? RateType.PER_CLIENT : RateType.OVERALL,
                rule.getCount(), rule.getPeriod().toMillis(), RateIntervalUnit.MILLISECONDS);
        //限流单元默认24小时后自动清理
        limiter.expireAsync(24, TimeUnit.HOURS);
        return limiter.tryAcquire();
    }
}
