package com.qmth.boot.api.exception;

import org.springframework.http.HttpStatus;

/**
 * 框架预留的异常类型，以及已使用的错误码
 */
public enum DefaultExceptionEnum {

    HEADER_INVALID(HttpStatus.BAD_REQUEST, 401001, "header field invalid"),
    PLATFORM_FORBIDDEN(HttpStatus.FORBIDDEN, 403001, "platform not allowed"),
    IP_FORBIDDEN(HttpStatus.FORBIDDEN, 403002, "ip not allowed"),
    AUTHORIZATION_ERROR(HttpStatus.UNAUTHORIZED, 401000, "authorization error"),
    AUTHORIZATION_FAILE(HttpStatus.UNAUTHORIZED, 401001, "authorization faile"),
    COMMON_EXCEPTION(HttpStatus.INTERNAL_SERVER_ERROR, 500000, "internal server error"),
    RATE_LIMITED(HttpStatus.SERVICE_UNAVAILABLE, 503001, "rate limited");

    private HttpStatus status;

    private int code;

    private String name;

    private DefaultExceptionEnum(HttpStatus status, int code, String name) {
        this.status = status;
        this.code = code;
        this.name = name;
    }

    public ApiException exception() {
        return new ApiException(status, code, name, null);
    }

    public ApiException exception(int code, String message) {
        return new ApiException(status, code, name, message);
    }

    public ApiException exception(String message) {
        return new ApiException(status, code, name, message);
    }

    public HttpStatus getStatus() {
        return status;
    }

    public int getCode() {
        return code;
    }

    public String getName() {
        return name;
    }
}
