package com.qmth.boot.api.exception;

import com.qmth.boot.core.exception.ReentrantException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.lang.Nullable;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.RestControllerAdvice;
import org.springframework.web.context.request.WebRequest;
import org.springframework.web.servlet.mvc.method.annotation.ResponseEntityExceptionHandler;

@RestControllerAdvice
public class GlobalExceptionHandler extends ResponseEntityExceptionHandler {

    protected static final Logger log = LoggerFactory.getLogger(GlobalExceptionHandler.class);

    @Override
    protected ResponseEntity<Object> handleExceptionInternal(Exception ex, @Nullable Object body, HttpHeaders headers,
            HttpStatus status, WebRequest request) {
        if (ex instanceof ApiException) {
            ApiException exception = (ApiException) ex;
            return new ResponseEntity<>(exception.responseEntity(), headers, exception.getStatus());
        } else {
            log.error(ex.getMessage(), ex);
            return new ResponseEntity<>(
                    new ExceptionResponseEntity(status.value(), status.name().toLowerCase(), ex.getMessage()), headers,
                    status);
        }
    }

    @ExceptionHandler(ApiException.class)
    public ResponseEntity<Object> handleApiException(ApiException e, WebRequest request) {
        return handleExceptionInternal(e, null, new HttpHeaders(), e.getStatus(), request);
    }

    @ExceptionHandler(ReentrantException.class)
    public ResponseEntity<Object> handleReentrantException(ReentrantException e, WebRequest request) {
        return handleExceptionInternal(e, null, new HttpHeaders(), HttpStatus.SERVICE_UNAVAILABLE, request);
    }

    @ExceptionHandler(Exception.class)
    public ResponseEntity<Object> handleOtherException(Exception e, WebRequest request) {
        return handleExceptionInternal(e, null, new HttpHeaders(), HttpStatus.INTERNAL_SERVER_ERROR, request);
    }

}
