package com.qmth.boot.tools.codec;

import org.apache.commons.lang3.StringUtils;

import javax.xml.bind.DatatypeConverter;
import java.nio.charset.StandardCharsets;
import java.security.MessageDigest;
import java.util.Base64;

public class CodecUtils {

    private static final String SHA1_ALGORITHM = "SHA-1";

    private static final String MD5_ALGORITHM = "MD5";

    public static String toBase64(byte[] value) {
        return value != null ? Base64.getEncoder().encodeToString(value) : StringUtils.EMPTY;
    }

    public static byte[] fromBase64(String value) {
        return value != null ? Base64.getDecoder().decode(value) : null;
    }

    public static String toHexString(byte[] value) {
        return value != null ? DatatypeConverter.printHexBinary(value) : StringUtils.EMPTY;
    }

    public static byte[] fromHexString(String value) {
        return value != null ? DatatypeConverter.parseHexBinary(value) : null;
    }

    public static byte[] sha1(String value) {
        return value != null ? digest(value.getBytes(StandardCharsets.UTF_8), SHA1_ALGORITHM) : null;
    }

    public static byte[] md5(String value) {
        return value != null ? digest(value.getBytes(StandardCharsets.UTF_8), MD5_ALGORITHM) : null;
    }

    public static byte[] digest(byte[] input, String algorithm) {
        try {
            return MessageDigest.getInstance(algorithm).digest(input);
        } catch (Exception e) {
            return null;
        }
    }

}
